import { concat } from 'uint8arrays';
export class Uint8ArrayList {
    constructor(data) {
        this.bufs = [];
        if (data != null) {
            this.bufs.push(data);
        }
        this.length = this.bufs.reduce((acc, curr) => acc + curr.byteLength, 0);
    }
    append(data) {
        this.bufs.push(data);
        this.length += data.byteLength;
    }
    get(index) {
        if (index == null || index < 0 || index >= this.length) {
            throw new RangeError('index is out of bounds');
        }
        let offset = 0;
        for (const buf of this.bufs) {
            const bufEnd = offset + buf.byteLength;
            if (index < bufEnd) {
                return buf[index - offset];
            }
            offset = bufEnd;
        }
        throw new RangeError('index is out of bounds');
    }
    /**
     * Remove bytes from the front of the pool
     */
    consume(bytes) {
        // first, normalize the argument, in accordance with how Buffer does it
        bytes = Math.trunc(bytes);
        // do nothing if not a positive number
        if (Number.isNaN(bytes) || bytes <= 0) {
            return;
        }
        while (this.bufs.length) {
            if (bytes >= this.bufs[0].byteLength) {
                bytes -= this.bufs[0].byteLength;
                this.length -= this.bufs[0].byteLength;
                this.bufs.shift();
            }
            else {
                this.bufs[0] = this.bufs[0].subarray(bytes);
                this.length -= bytes;
                break;
            }
        }
    }
    toUint8Array(beginInclusive = 0, endExclusive) {
        endExclusive = endExclusive ?? (this.length > 0 ? this.length : 0);
        if (beginInclusive < 0 || endExclusive > this.length) {
            throw new RangeError('index out of bounds');
        }
        let bufs = [];
        let offset = 0;
        for (const buf of this.bufs) {
            const bufStart = offset;
            const bufEnd = bufStart + buf.byteLength;
            const sliceStartInBuf = beginInclusive >= bufStart && beginInclusive < bufEnd;
            const sliceEndsInBuf = endExclusive > bufStart && endExclusive <= bufEnd;
            const bufInSlice = beginInclusive < bufStart && endExclusive >= bufEnd;
            offset = bufEnd;
            let startIndex;
            let endIndex;
            if (sliceStartInBuf) {
                startIndex = beginInclusive - bufStart;
                endIndex = buf.byteLength;
            }
            if (sliceEndsInBuf) {
                endIndex = endExclusive - bufStart;
                if (startIndex == null) {
                    startIndex = 0;
                }
            }
            if (bufInSlice) {
                startIndex = 0;
                endIndex = buf.byteLength;
            }
            if (startIndex != null && endIndex != null) {
                bufs.push(buf.subarray(startIndex, endIndex));
            }
            if (sliceEndsInBuf) {
                break;
            }
        }
        return concat(bufs, endExclusive - beginInclusive);
    }
}
//# sourceMappingURL=index.js.map